@@ -8,8 +8,10 @@
 
 #include "BKE_cryptomatte.h"
 #include "BKE_cryptomatte.hh"
-#include "BKE_image.h"
-#include "BKE_main.h"
+#include "BKE_image.hh"
+#include "BKE_layer.hh"
+#include "BKE_main.hh"
+#include "BKE_material.hh"
 
 #include "DNA_layer_types.h"
 #include "DNA_material_types.h"
@@ -17,9 +19,8 @@
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 
-#include "BLI_compiler_attrs.h"
 #include "BLI_dynstr.h"
-#include "BLI_hash_mm3.h"
+#include "BLI_hash_mm3.hh"
 #include "BLI_listbase.h"
 #include "BLI_string.h"
 
@@ -32,7 +33,6 @@
 #include <iomanip>
 #include <sstream>
 #include <string>
-#include <string_view>
 
 struct CryptomatteSession {
   blender::Map<std::string, blender::bke::cryptomatte::CryptomatteLayer> layers;
@@ -43,15 +43,13 @@ struct CryptomatteSession {
   CryptomatteSession(const Main *bmain);
   CryptomatteSession(StampData *stamp_data);
   CryptomatteSession(const ViewLayer *view_layer);
-  CryptomatteSession(const Scene *scene);
-  void init(const ViewLayer *view_layer);
+  CryptomatteSession(const Scene *scene, bool build_meta_data = false);
+  void init(const ViewLayer *view_layer, bool build_meta_data = false);
 
   blender::bke::cryptomatte::CryptomatteLayer &add_layer(std::string layer_name);
   std::optional<std::string> operator[](float encoded_hash) const;
 
-#ifdef WITH_CXX_GUARDEDALLOC
   MEM_CXX_CLASS_ALLOC_FUNCS("cryptomatte:CryptomatteSession")
-#endif
 };
 
 CryptomatteSession::CryptomatteSession(const Main *bmain)
@@ -93,29 +91,57 @@ CryptomatteSession::CryptomatteSession(const ViewLayer *view_layer)
   init(view_layer);
 }
 
-CryptomatteSession::CryptomatteSession(const Scene *scene)
+CryptomatteSession::CryptomatteSession(const Scene *scene, bool build_meta_data)
 {
+
+  if (build_meta_data) {
+    BKE_scene_view_layers_synced_ensure(scene);
+  }
+
   LISTBASE_FOREACH (const ViewLayer *, view_layer, &scene->view_layers) {
-    init(view_layer);
+    init(view_layer, build_meta_data);
   }
 }
 
-void CryptomatteSession::init(const ViewLayer *view_layer)
+void CryptomatteSession::init(const ViewLayer *view_layer, bool build_meta_data)
 {
   eViewLayerCryptomatteFlags cryptoflags = static_cast<eViewLayerCryptomatteFlags>(
       view_layer->cryptomatte_flag & VIEW_LAYER_CRYPTOMATTE_ALL);
   if (cryptoflags == 0) {
-    cryptoflags = static_cast<eViewLayerCryptomatteFlags>(VIEW_LAYER_CRYPTOMATTE_ALL);
+    cryptoflags = VIEW_LAYER_CRYPTOMATTE_ALL;
   }
 
+  ListBase *object_bases = BKE_view_layer_object_bases_get(const_cast<ViewLayer *>(view_layer));
+
   if (cryptoflags & VIEW_LAYER_CRYPTOMATTE_OBJECT) {
-    add_layer(blender::StringRefNull(view_layer->name) + "." + RE_PASSNAME_CRYPTOMATTE_OBJECT);
+    blender::bke::cryptomatte::CryptomatteLayer &objects = add_layer(
+        blender::StringRefNull(view_layer->name) + "." + RE_PASSNAME_CRYPTOMATTE_OBJECT);
+
+    if (build_meta_data) {
+      LISTBASE_FOREACH (Base *, base, object_bases) {
+        objects.add_ID(base->object->id);
+      }
+    }
   }
+
   if (cryptoflags & VIEW_LAYER_CRYPTOMATTE_ASSET) {
     add_layer(blender::StringRefNull(view_layer->name) + "." + RE_PASSNAME_CRYPTOMATTE_ASSET);
   }
+
   if (cryptoflags & VIEW_LAYER_CRYPTOMATTE_MATERIAL) {
-    add_layer(blender::StringRefNull(view_layer->name) + "." + RE_PASSNAME_CRYPTOMATTE_MATERIAL);
+    blender::bke::cryptomatte::CryptomatteLayer &materials = add_layer(
+        blender::StringRefNull(view_layer->name) + "." + RE_PASSNAME_CRYPTOMATTE_MATERIAL);
+
+    if (build_meta_data) {
+      LISTBASE_FOREACH (Base *, base, object_bases) {
+        for (int i = 0; i < base->object->totcol; i++) {
+          Material *material = BKE_object_material_get(base->object, i + 1);
+          if (material) {
+            materials.add_ID(material->id);
+          }
+        }
+      }
+    }
   }
 }
 
@@ -150,9 +176,9 @@ CryptomatteSession *BKE_cryptomatte_init_from_render_result(const RenderResult *
   return session;
 }
 
-CryptomatteSession *BKE_cryptomatte_init_from_scene(const Scene *scene)
+CryptomatteSession *BKE_cryptomatte_init_from_scene(const Scene *scene, bool build_meta_data)
 {
-  CryptomatteSession *session = new CryptomatteSession(scene);
+  CryptomatteSession *session = new CryptomatteSession(scene, build_meta_data);
   return session;
 }
 
@@ -238,7 +264,7 @@ char *BKE_cryptomatte_entries_to_matte_id(NodeCryptomatte *node_storage)
     if (!first) {
       BLI_dynstr_append(matte_id, ",");
     }
-    if (BLI_strnlen(entry->name, sizeof(entry->name)) != 0) {
+    if (STRNLEN(entry->name) != 0) {
       BLI_dynstr_nappend(matte_id, entry->name, sizeof(entry->name));
     }
     else {
@@ -283,13 +309,13 @@ void BKE_cryptomatte_matte_id_to_entries(NodeCryptomatte *node_storage, const ch
       token = token.substr(first, (last - first + 1));
       if (*token.begin() == '<' && *(--token.end()) == '>') {
         float encoded_hash = atof(token.substr(1, token.length() - 2).c_str());
-        entry = MEM_cnew<CryptomatteEntry>(__func__);
+        entry = MEM_callocN<CryptomatteEntry>(__func__);
         entry->encoded_hash = encoded_hash;
       }
       else {
         const char *name = token.c_str();
         int name_len = token.length();
-        entry = MEM_cnew<CryptomatteEntry>(__func__);
+        entry = MEM_callocN<CryptomatteEntry>(__func__);
         STRNCPY(entry->name, name);
         uint32_t hash = BKE_cryptomatte_hash(name, name_len);
         entry->encoded_hash = BKE_cryptomatte_hash_to_float(hash);
@@ -301,15 +327,6 @@ void BKE_cryptomatte_matte_id_to_entries(NodeCryptomatte *node_storage, const ch
   }
 }
 
-static std::string cryptomatte_determine_name(const ViewLayer *view_layer,
-                                              const blender::StringRefNull cryptomatte_layer_name)
-{
-  std::stringstream stream;
-  const size_t view_layer_name_len = BLI_strnlen(view_layer->name, sizeof(view_layer->name));
-  stream << std::string(view_layer->name, view_layer_name_len) << "." << cryptomatte_layer_name;
-  return stream.str();
-}
-
 static uint32_t cryptomatte_determine_identifier(const blender::StringRef name)
 {
   return BLI_hash_mm3(reinterpret_cast<const uchar *>(name.data()), name.size(), 0);
@@ -326,9 +343,7 @@ static void add_render_result_meta_data(RenderResult *render_result,
       value.data());
 }
 
-void BKE_cryptomatte_store_metadata(const CryptomatteSession *session,
-                                    RenderResult *render_result,
-                                    const ViewLayer *view_layer)
+void BKE_cryptomatte_store_metadata(const CryptomatteSession *session, RenderResult *render_result)
 {
   for (const blender::MapItem<std::string, blender::bke::cryptomatte::CryptomatteLayer> item :
        session->layers.items())
@@ -337,12 +352,11 @@ void BKE_cryptomatte_store_metadata(const CryptomatteSession *session,
     const blender::bke::cryptomatte::CryptomatteLayer &layer = item.value;
 
     const std::string manifest = layer.manifest();
-    const std::string name = cryptomatte_determine_name(view_layer, layer_name);
 
-    add_render_result_meta_data(render_result, name, "name", name);
-    add_render_result_meta_data(render_result, name, "hash", "MurmurHash3_32");
-    add_render_result_meta_data(render_result, name, "conversion", "uint32_to_float32");
-    add_render_result_meta_data(render_result, name, "manifest", manifest);
+    add_render_result_meta_data(render_result, layer_name, "name", layer_name);
+    add_render_result_meta_data(render_result, layer_name, "hash", "MurmurHash3_32");
+    add_render_result_meta_data(render_result, layer_name, "conversion", "uint32_to_float32");
+    add_render_result_meta_data(render_result, layer_name, "manifest", manifest);
   }
 }
 
