@@ -2,6 +2,10 @@
  *
  * SPDX-License-Identifier: GPL-2.0-or-later */
 
+/** \file
+ * \ingroup bli
+ */
+
 #include <fmt/format.h>
 #include <iostream>
 
@@ -16,6 +20,7 @@
 #include "BLI_index_mask_expression.hh"
 #include "BLI_index_ranges_builder.hh"
 #include "BLI_math_base.hh"
+#include "BLI_rand.hh"
 #include "BLI_set.hh"
 #include "BLI_sort.hh"
 #include "BLI_task.hh"
@@ -586,6 +591,11 @@ IndexMask IndexMask::from_bools(const VArray<bool> &bools, IndexMaskMemory &memo
   return IndexMask::from_bools(bools.index_range(), bools, memory);
 }
 
+IndexMask IndexMask::from_bools_inverse(const VArray<bool> &bools, IndexMaskMemory &memory)
+{
+  return IndexMask::from_bools_inverse(bools.index_range(), bools, memory);
+}
+
 IndexMask IndexMask::from_bools(const IndexMask &universe,
                                 Span<bool> bools,
                                 IndexMaskMemory &memory)
@@ -640,6 +650,22 @@ IndexMask IndexMask::from_bools(const IndexMask &universe,
       universe, GrainSize(512), memory, [&](const int64_t index) { return bools[index]; });
 }
 
+IndexMask IndexMask::from_bools_inverse(const IndexMask &universe,
+                                        const VArray<bool> &bools,
+                                        IndexMaskMemory &memory)
+{
+  const CommonVArrayInfo info = bools.common_info();
+  if (info.type == CommonVArrayInfo::Type::Single) {
+    return *static_cast<const bool *>(info.data) ? IndexMask() : universe;
+  }
+  if (info.type == CommonVArrayInfo::Type::Span) {
+    const Span<bool> span(static_cast<const bool *>(info.data), bools.size());
+    return IndexMask::from_bools_inverse(universe, span, memory);
+  }
+  return IndexMask::from_predicate(
+      universe, GrainSize(512), memory, [&](const int64_t index) { return !bools[index]; });
+}
+
 template<typename T>
 IndexMask IndexMask::from_ranges(OffsetIndices<T> offsets,
                                  const IndexMask &mask,
@@ -1206,4 +1232,29 @@ template IndexMask IndexMask::from_ranges(OffsetIndices<int64_t>,
                                           const IndexMask &,
                                           IndexMaskMemory &);
 
+IndexMask random_mask(const IndexMask &mask,
+                      const int64_t universe_size,
+                      const uint32_t random_seed,
+                      const float probability,
+                      IndexMaskMemory &memory)
+{
+  RandomNumberGenerator rng{random_seed};
+  const auto next_bool_random_value = [&]() { return rng.get_float() <= probability; };
+
+  Array<bool> random(universe_size, false);
+  mask.foreach_index_optimized<int64_t>(
+      [&](const int64_t i) { random[i] = next_bool_random_value(); });
+
+  return IndexMask::from_bools(IndexRange(universe_size), random, memory);
+}
+
+IndexMask random_mask(const int64_t universe_size,
+                      const uint32_t random_seed,
+                      const float probability,
+                      IndexMaskMemory &memory)
+{
+  const IndexRange selection(universe_size);
+  return random_mask(selection, universe_size, random_seed, probability, memory);
+}
+
 }  // namespace blender::index_mask
