@@ -92,7 +92,7 @@ static void arrow_draw_geom(const ArrowGizmo3D *arrow,
   immUniform2fv("viewportSize", &viewport[2]);
 
   if (draw_style == ED_GIZMO_ARROW_STYLE_CROSS) {
-    immUniform1f("lineWidth", U.pixelsize);
+    immUniform1f("lineWidth", U.pixelsize + WM_gizmo_select_bias(select));
     immUniformColor4fv(color);
 
     immBegin(GPU_PRIM_LINES, 4);
@@ -114,11 +114,13 @@ static void arrow_draw_geom(const ArrowGizmo3D *arrow,
         {-unitx, unity, 0},
     };
 
-    immUniform1f("lineWidth", arrow->gizmo.line_width * U.pixelsize);
+    immUniform1f("lineWidth",
+                 (arrow->gizmo.line_width * U.pixelsize) + WM_gizmo_select_bias(select));
     wm_gizmo_vec_draw(color, vec, ARRAY_SIZE(vec), pos, GPU_PRIM_LINE_LOOP);
   }
   else if (draw_style == ED_GIZMO_ARROW_STYLE_PLANE) {
-    const float scale = 0.1f;
+    /* Increase the size a bit during selection. These are relatively easy to hit. */
+    const float scale = select ? 0.15f : 0.1f;
     const float verts[4][3] = {
         {0, 0, 0},
         {scale, 0, scale},
@@ -132,7 +134,8 @@ static void arrow_draw_geom(const ArrowGizmo3D *arrow,
     GPU_matrix_push();
     GPU_matrix_translate_3f(0.0f, 0.0f, arrow_length);
 
-    immUniform1f("lineWidth", arrow->gizmo.line_width * U.pixelsize);
+    immUniform1f("lineWidth",
+                 (arrow->gizmo.line_width * U.pixelsize) + WM_gizmo_select_bias(select));
     wm_gizmo_vec_draw(color, verts, ARRAY_SIZE(verts), pos, GPU_PRIM_LINE_LOOP);
 
     immUnbindProgram();
@@ -150,9 +153,8 @@ static void arrow_draw_geom(const ArrowGizmo3D *arrow,
     };
 
     if (draw_options & ED_GIZMO_ARROW_DRAW_FLAG_STEM) {
-      const float stem_width = arrow->gizmo.line_width * U.pixelsize +
-                               (select ? ARROW_SELECT_THRESHOLD_PX * UI_SCALE_FAC : 0);
-      immUniform1f("lineWidth", stem_width);
+      immUniform1f("lineWidth",
+                   (arrow->gizmo.line_width * U.pixelsize) + WM_gizmo_select_bias(select));
       wm_gizmo_vec_draw(color, vec, ARRAY_SIZE(vec), pos, GPU_PRIM_LINE_STRIP);
     }
     else {
@@ -166,7 +168,8 @@ static void arrow_draw_geom(const ArrowGizmo3D *arrow,
     /* NOTE: ideally #ARROW_SELECT_THRESHOLD_PX would be added here, however adding a
      * margin in pixel space isn't so simple, nor is it as important as for the arrow stem. */
     if (draw_style == ED_GIZMO_ARROW_STYLE_BOX) {
-      const float size = 0.05f;
+      /* Increase the size during selection so it is wider than other lines. */
+      const float size = select ? 0.11f : 0.05f;
 
       /* translate to line end with some extra offset so box starts exactly where line ends */
       GPU_matrix_translate_3f(0.0f, 0.0f, arrow_length + size);
@@ -181,8 +184,9 @@ static void arrow_draw_geom(const ArrowGizmo3D *arrow,
     else {
       BLI_assert(draw_style == ED_GIZMO_ARROW_STYLE_NORMAL);
 
-      const float len = 0.25f;
-      const float width = 0.06f;
+      /* Increase the size during selection, but mostly wider. */
+      const float len = select ? 0.35f : 0.25f;
+      const float width = select ? 0.12f : 0.06f;
 
       /* translate to line end */
       GPU_matrix_translate_3f(0.0f, 0.0f, arrow_length);
@@ -320,10 +324,10 @@ static int gizmo_arrow_test_select(bContext * /*C*/, wmGizmo *gz, const int mval
  * Calculate arrow offset independent from prop min value,
  * meaning the range will not be offset by min value first.
  */
-static int gizmo_arrow_modal(bContext *C,
-                             wmGizmo *gz,
-                             const wmEvent *event,
-                             eWM_GizmoFlagTweak tweak_flag)
+static wmOperatorStatus gizmo_arrow_modal(bContext *C,
+                                          wmGizmo *gz,
+                                          const wmEvent *event,
+                                          eWM_GizmoFlagTweak tweak_flag)
 {
   if (event->type != MOUSEMOVE) {
     return OPERATOR_RUNNING_MODAL;
@@ -421,7 +425,7 @@ static void gizmo_arrow_setup(wmGizmo *gz)
   arrow->data.range_fac = 1.0f;
 }
 
-static int gizmo_arrow_invoke(bContext * /*C*/, wmGizmo *gz, const wmEvent *event)
+static wmOperatorStatus gizmo_arrow_invoke(bContext * /*C*/, wmGizmo *gz, const wmEvent *event)
 {
   ArrowGizmo3D *arrow = (ArrowGizmo3D *)gz;
   GizmoInteraction *inter = static_cast<GizmoInteraction *>(
@@ -522,7 +526,7 @@ static void GIZMO_GT_arrow_3d(wmGizmoType *gzt)
   /* identifiers */
   gzt->idname = "GIZMO_GT_arrow_3d";
 
-  /* api callbacks */
+  /* API callbacks. */
   gzt->draw = gizmo_arrow_draw;
   gzt->draw_select = gizmo_arrow_draw_select;
   gzt->test_select = gizmo_arrow_test_select;
