@@ -9,13 +9,14 @@
 #include <condition_variable>
 #include <cstddef>
 #include <cstring>
-#include <mutex>
 
 #include "MEM_guardedalloc.h"
 
 #include "BLI_bitmap.h"
 #include "BLI_fileops.h"
+#include "BLI_listbase.h"
 #include "BLI_math_color_blend.h"
+#include "BLI_mutex.hh"
 #include "BLI_string.h"
 #include "BLI_task.h"
 #include "BLI_task.hh"
@@ -79,6 +80,8 @@
 
 #include "render_intern.hh"
 
+namespace path_templates = blender::bke::path_templates;
+
 /* TODO(sergey): Find better approximation of the scheduled frames.
  * For really high-resolution renders it might fail still. */
 #define MAX_SCHEDULED_FRAMES 8
@@ -112,7 +115,7 @@ struct OGLRender : public RenderJobBase {
 
   GPUViewport *viewport = nullptr;
 
-  std::mutex reports_mutex;
+  blender::Mutex reports_mutex;
   ReportList *reports = nullptr;
 
   int cfrao = 0;
@@ -180,7 +183,7 @@ static void screen_opengl_views_setup(OGLRender *oglrender)
     rv = static_cast<RenderView *>(rr->views.first);
 
     if (rv == nullptr) {
-      rv = MEM_cnew<RenderView>("new opengl render view");
+      rv = MEM_callocN<RenderView>("new opengl render view");
       BLI_addtail(&rr->views, rv);
     }
 
@@ -228,7 +231,7 @@ static void screen_opengl_views_setup(OGLRender *oglrender)
           BLI_findstring(&rr->views, srv->name, offsetof(SceneRenderView, name)));
 
       if (rv == nullptr) {
-        rv = MEM_cnew<RenderView>("new opengl render view");
+        rv = MEM_callocN<RenderView>("new opengl render view");
         STRNCPY(rv->name, srv->name);
         BLI_addtail(&rr->views, rv);
       }
@@ -277,14 +280,14 @@ static void screen_opengl_render_doit(OGLRender *oglrender, RenderResult *rr)
        * it actually makes sense to keep float buffer instead.
        */
       if (ibuf_result->float_buffer.data != nullptr) {
-        IMB_rect_from_float(ibuf_result);
-        imb_freerectfloatImBuf(ibuf_result);
+        IMB_byte_from_float(ibuf_result);
+        IMB_free_float_pixels(ibuf_result);
       }
       BLI_assert((sizex == ibuf->x) && (sizey == ibuf->y));
     }
     else if (gpd) {
       /* If there are no strips, Grease Pencil still needs a buffer to draw on */
-      ibuf_result = IMB_allocImBuf(sizex, sizey, 32, IB_rect);
+      ibuf_result = IMB_allocImBuf(sizex, sizey, 32, IB_byte_data);
     }
 
     if (gpd) {
@@ -340,7 +343,7 @@ static void screen_opengl_render_doit(OGLRender *oglrender, RenderResult *rr)
                                                  region,
                                                  sizex,
                                                  sizey,
-                                                 IB_rectfloat,
+                                                 IB_float_data,
                                                  alpha_mode,
                                                  viewname,
                                                  true,
@@ -361,7 +364,7 @@ static void screen_opengl_render_doit(OGLRender *oglrender, RenderResult *rr)
                                                         scene->camera,
                                                         sizex,
                                                         sizey,
-                                                        IB_rectfloat,
+                                                        IB_float_data,
                                                         V3D_OFSDRAW_SHOW_ANNOTATION,
                                                         alpha_mode,
                                                         viewname,
@@ -414,20 +417,32 @@ static void screen_opengl_render_write(OGLRender *oglrender)
 
   rr = RE_AcquireResultRead(oglrender->re);
 
-  BKE_image_path_from_imformat(filepath,
-                               scene->r.pic,
-                               BKE_main_blendfile_path(oglrender->bmain),
-                               scene->r.cfra,
-                               &scene->r.im_format,
-                               (scene->r.scemode & R_EXTENSION) != 0,
-                               false,
-                               nullptr);
+  const char *relbase = BKE_main_blendfile_path(oglrender->bmain);
+  const path_templates::VariableMap template_variables =
+      BKE_build_template_variables_for_render_path(&scene->r);
+  const blender::Vector<path_templates::Error> errors = BKE_image_path_from_imformat(
+      filepath,
+      scene->r.pic,
+      relbase,
+      &template_variables,
+      scene->r.cfra,
+      &scene->r.im_format,
+      (scene->r.scemode & R_EXTENSION) != 0,
+      false,
+      nullptr);
 
-  /* write images as individual images or stereo */
-  BKE_render_result_stamp_info(scene, scene->camera, rr, false);
-  ok = BKE_image_render_write(oglrender->reports, rr, scene, false, filepath);
+  if (!errors.is_empty()) {
+    std::unique_lock lock(oglrender->reports_mutex);
+    BKE_report_path_template_errors(oglrender->reports, RPT_ERROR, scene->r.pic, errors);
+    ok = false;
+  }
+  else {
+    /* write images as individual images or stereo */
+    BKE_render_result_stamp_info(scene, scene->camera, rr, false);
+    ok = BKE_image_render_write(oglrender->reports, rr, scene, false, filepath);
 
-  RE_ReleaseResultImage(oglrender->re);
+    RE_ReleaseResultImage(oglrender->re);
+  }
 
   if (ok) {
     printf("OpenGL Render written to '%s'\n", filepath);
@@ -461,24 +476,24 @@ static void screen_opengl_render_apply(OGLRender *oglrender)
   if (oglrender->is_sequencer) {
     Scene *scene = oglrender->scene;
 
-    SeqRenderData context;
+    blender::seq::RenderData context;
     SpaceSeq *sseq = oglrender->sseq;
     int chanshown = sseq ? sseq->chanshown : 0;
 
-    SEQ_render_new_render_data(oglrender->bmain,
-                               oglrender->depsgraph,
-                               scene,
-                               oglrender->sizex,
-                               oglrender->sizey,
-                               SEQ_RENDER_SIZE_SCENE,
-                               false,
-                               &context);
+    blender::seq::render_new_render_data(oglrender->bmain,
+                                         oglrender->depsgraph,
+                                         scene,
+                                         oglrender->sizex,
+                                         oglrender->sizey,
+                                         SEQ_RENDER_SIZE_SCENE,
+                                         false,
+                                         &context);
 
     for (view_id = 0; view_id < oglrender->views_len; view_id++) {
       context.view_id = view_id;
       context.gpu_offscreen = oglrender->ofs;
       context.gpu_viewport = oglrender->viewport;
-      oglrender->seq_data.ibufs_arr[view_id] = SEQ_render_give_ibuf(
+      oglrender->seq_data.ibufs_arr[view_id] = blender::seq::render_give_ibuf(
           &context, scene->r.cfra, chanshown);
     }
   }
@@ -753,6 +768,7 @@ static bool screen_opengl_render_init(bContext *C, wmOperator *op)
                              true,
                              GPU_RGBA16F,
                              GPU_TEXTURE_USAGE_SHADER_READ | GPU_TEXTURE_USAGE_HOST_READ,
+                             false,
                              err_out);
   DRW_gpu_context_disable();
 
@@ -806,9 +822,7 @@ static bool screen_opengl_render_init(bContext *C, wmOperator *op)
     oglrender->rv3d = static_cast<RegionView3D *>(oglrender->region->regiondata);
 
     /* MUST be cleared on exit */
-    memset(&oglrender->scene->customdata_mask_modal,
-           0,
-           sizeof(oglrender->scene->customdata_mask_modal));
+    oglrender->scene->customdata_mask_modal = CustomData_MeshMasks{};
     ED_view3d_datamask(oglrender->scene,
                        oglrender->view_layer,
                        oglrender->v3d,
@@ -912,9 +926,7 @@ static void screen_opengl_render_end(OGLRender *oglrender)
 
   MEM_SAFE_FREE(oglrender->seq_data.ibufs_arr);
 
-  memset(&oglrender->scene->customdata_mask_modal,
-         0,
-         sizeof(oglrender->scene->customdata_mask_modal));
+  oglrender->scene->customdata_mask_modal = CustomData_MeshMasks{};
 
   if (oglrender->wm_job) { /* exec will not have a job */
     Depsgraph *depsgraph = oglrender->depsgraph;
@@ -1035,17 +1047,29 @@ static void write_result(TaskPool *__restrict pool, WriteTaskData *task_data)
      * calculate file name again here.
      */
     char filepath[FILE_MAX];
-    BKE_image_path_from_imformat(filepath,
-                                 scene->r.pic,
-                                 BKE_main_blendfile_path(oglrender->bmain),
-                                 cfra,
-                                 &scene->r.im_format,
-                                 (scene->r.scemode & R_EXTENSION) != 0,
-                                 true,
-                                 nullptr);
+    const char *relbase = BKE_main_blendfile_path(oglrender->bmain);
+    const path_templates::VariableMap template_variables =
+        BKE_build_template_variables_for_render_path(&scene->r);
+    const blender::Vector<path_templates::Error> errors = BKE_image_path_from_imformat(
+        filepath,
+        scene->r.pic,
+        relbase,
+        &template_variables,
+        cfra,
+        &scene->r.im_format,
+        (scene->r.scemode & R_EXTENSION) != 0,
+        true,
+        nullptr);
+
+    if (!errors.is_empty()) {
+      BKE_report_path_template_errors(&reports, RPT_ERROR, scene->r.pic, errors);
+      ok = false;
+    }
+    else {
+      BKE_render_result_stamp_info(scene, scene->camera, rr, false);
+      ok = BKE_image_render_write(nullptr, rr, scene, true, filepath);
+    }
 
-    BKE_render_result_stamp_info(scene, scene->camera, rr, false);
-    ok = BKE_image_render_write(nullptr, rr, scene, true, filepath);
     if (!ok) {
       BKE_reportf(&reports, RPT_ERROR, "Write error: cannot save %s", filepath);
     }
@@ -1088,7 +1112,7 @@ static bool schedule_write_result(OGLRender *oglrender, RenderResult *rr)
     return false;
   }
   Scene *scene = oglrender->scene;
-  WriteTaskData *task_data = MEM_cnew<WriteTaskData>("write task data");
+  WriteTaskData *task_data = MEM_callocN<WriteTaskData>("write task data");
   task_data->rr = rr;
   memcpy(&task_data->tmp_scene, scene, sizeof(task_data->tmp_scene));
   {
@@ -1124,16 +1148,26 @@ static bool screen_opengl_render_anim_step(OGLRender *oglrender)
   is_movie = BKE_imtype_is_movie(scene->r.im_format.imtype);
 
   if (!is_movie) {
-    BKE_image_path_from_imformat(filepath,
-                                 scene->r.pic,
-                                 BKE_main_blendfile_path(oglrender->bmain),
-                                 scene->r.cfra,
-                                 &scene->r.im_format,
-                                 (scene->r.scemode & R_EXTENSION) != 0,
-                                 true,
-                                 nullptr);
+    const char *relbase = BKE_main_blendfile_path(oglrender->bmain);
+    const path_templates::VariableMap template_variables =
+        BKE_build_template_variables_for_render_path(&scene->r);
+    const blender::Vector<path_templates::Error> errors = BKE_image_path_from_imformat(
+        filepath,
+        scene->r.pic,
+        relbase,
+        &template_variables,
+        scene->r.cfra,
+        &scene->r.im_format,
+        (scene->r.scemode & R_EXTENSION) != 0,
+        true,
+        nullptr);
 
-    if ((scene->r.mode & R_NO_OVERWRITE) && BLI_exists(filepath)) {
+    if (!errors.is_empty()) {
+      std::unique_lock lock(oglrender->reports_mutex);
+      BKE_report_path_template_errors(oglrender->reports, RPT_ERROR, scene->r.pic, errors);
+      ok = false;
+    }
+    else if ((scene->r.mode & R_NO_OVERWRITE) && BLI_exists(filepath)) {
       {
         std::unique_lock lock(oglrender->reports_mutex);
         BKE_reportf(oglrender->reports, RPT_INFO, "Skipping existing frame \"%s\"", filepath);
@@ -1195,7 +1229,9 @@ finally: /* Step the frame and bail early if needed */
   return true;
 }
 
-static int screen_opengl_render_modal(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus screen_opengl_render_modal(bContext *C,
+                                                   wmOperator *op,
+                                                   const wmEvent *event)
 {
   OGLRender *oglrender = static_cast<OGLRender *>(op->customdata);
 
@@ -1260,7 +1296,9 @@ static void opengl_render_freejob(void *customdata)
   screen_opengl_render_end(oglrender);
 }
 
-static int screen_opengl_render_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus screen_opengl_render_invoke(bContext *C,
+                                                    wmOperator *op,
+                                                    const wmEvent *event)
 {
   const bool anim = RNA_boolean_get(op->ptr, "animation");
 
@@ -1304,7 +1342,7 @@ static int screen_opengl_render_invoke(bContext *C, wmOperator *op, const wmEven
 }
 
 /* executes blocking render */
-static int screen_opengl_render_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus screen_opengl_render_exec(bContext *C, wmOperator *op)
 {
   if (!screen_opengl_render_init(C, op)) {
     return OPERATOR_CANCELLED;
@@ -1362,7 +1400,7 @@ void RENDER_OT_opengl(wmOperatorType *ot)
   ot->description = "Take a snapshot of the active viewport";
   ot->idname = "RENDER_OT_opengl";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->get_description = screen_opengl_render_get_description;
   ot->invoke = screen_opengl_render_invoke;
   ot->exec = screen_opengl_render_exec; /* blocking */
