@@ -16,6 +16,7 @@
 
 #include "MEM_guardedalloc.h"
 
+#include "BLI_listbase.h"
 #include "BLI_utildefines.h"
 
 #include "BLT_translation.hh"
@@ -23,6 +24,7 @@
 #include "BKE_context.hh"
 #include "BKE_fcurve.hh"
 #include "BKE_lib_id.hh"
+#include "BKE_library.hh"
 #include "BKE_main.hh"
 #include "BKE_nla.hh"
 #include "BKE_report.hh"
@@ -95,7 +97,7 @@ void ED_nla_postop_refresh(bAnimContext *ac)
 /** \name Enable Tweak-Mode Operator
  * \{ */
 
-static int nlaedit_enable_tweakmode_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus nlaedit_enable_tweakmode_exec(bContext *C, wmOperator *op)
 {
   bAnimContext ac;
 
@@ -183,7 +185,7 @@ void NLA_OT_tweakmode_enter(wmOperatorType *ot)
   ot->description =
       "Enter tweaking mode for the action referenced by the active strip to edit its keyframes";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_enable_tweakmode_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -261,7 +263,7 @@ bool nlaedit_disable_tweakmode(bAnimContext *ac, bool do_solo)
 }
 
 /* Exit tweak-mode operator callback. */
-static int nlaedit_disable_tweakmode_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus nlaedit_disable_tweakmode_exec(bContext *C, wmOperator *op)
 {
   bAnimContext ac;
 
@@ -292,7 +294,7 @@ void NLA_OT_tweakmode_exit(wmOperatorType *ot)
   ot->idname = "NLA_OT_tweakmode_exit";
   ot->description = "Exit tweaking mode for the action referenced by the active strip";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_disable_tweakmode_exec;
   ot->poll = nlaop_poll_tweakmode_on;
 
@@ -373,7 +375,7 @@ static void get_nlastrip_extents(bAnimContext *ac, float *min, float *max, const
 /** \name Automatic Preview-Range Operator
  * \{ */
 
-static int nlaedit_previewrange_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_previewrange_exec(bContext *C, wmOperator * /*op*/)
 {
   bAnimContext ac;
   Scene *scene;
@@ -410,7 +412,7 @@ void NLA_OT_previewrange_set(wmOperatorType *ot)
   ot->idname = "NLA_OT_previewrange_set";
   ot->description = "Set Preview Range based on extends of selected strips";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_previewrange_exec;
   ot->poll = ED_operator_nla_active;
 
@@ -478,7 +480,7 @@ static bool nla_tracks_get_selected_extents(bAnimContext *ac, float *r_min, floa
   return (found != 0);
 }
 
-static int nlaedit_viewall(bContext *C, const bool only_sel)
+static wmOperatorStatus nlaedit_viewall(bContext *C, const bool only_sel)
 {
   bAnimContext ac;
   View2D *v2d;
@@ -530,13 +532,13 @@ static int nlaedit_viewall(bContext *C, const bool only_sel)
 
 /* ......... */
 
-static int nlaedit_viewall_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_viewall_exec(bContext *C, wmOperator * /*op*/)
 {
   /* whole range */
   return nlaedit_viewall(C, false);
 }
 
-static int nlaedit_viewsel_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_viewsel_exec(bContext *C, wmOperator * /*op*/)
 {
   /* only selected */
   return nlaedit_viewall(C, true);
@@ -549,7 +551,7 @@ void NLA_OT_view_all(wmOperatorType *ot)
   ot->idname = "NLA_OT_view_all";
   ot->description = "Reset viewable area to show full strips range";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_viewall_exec;
   ot->poll = ED_operator_nla_active;
 
@@ -564,7 +566,7 @@ void NLA_OT_view_selected(wmOperatorType *ot)
   ot->idname = "NLA_OT_view_selected";
   ot->description = "Reset viewable area to show selected strips range";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_viewsel_exec;
   ot->poll = ED_operator_nla_active;
 
@@ -578,7 +580,7 @@ void NLA_OT_view_selected(wmOperatorType *ot)
 /** \name View-Frame Operator
  * \{ */
 
-static int nlaedit_viewframe_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus nlaedit_viewframe_exec(bContext *C, wmOperator *op)
 {
   const int smooth_viewtx = WM_operator_smooth_viewtx_get(op);
   ANIM_center_frame(C, smooth_viewtx);
@@ -592,7 +594,7 @@ void NLA_OT_view_frame(wmOperatorType *ot)
   ot->idname = "NLA_OT_view_frame";
   ot->description = "Move the view to the current frame";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_viewframe_exec;
   ot->poll = ED_operator_nla_active;
 
@@ -619,7 +621,9 @@ static int nlaedit_get_editable_tracks(bAnimContext *ac, ListBase *anim_data)
   return ANIM_animdata_filter(ac, anim_data, filter, ac->data, eAnimCont_Types(ac->datatype));
 }
 
-static int nlaedit_add_actionclip_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+static wmOperatorStatus nlaedit_add_actionclip_invoke(bContext *C,
+                                                      wmOperator *op,
+                                                      const wmEvent *event)
 {
   /* Get editor data. */
   bAnimContext ac;
@@ -642,7 +646,7 @@ static int nlaedit_add_actionclip_invoke(bContext *C, wmOperator *op, const wmEv
 }
 
 /* add the specified action as new strip */
-static int nlaedit_add_actionclip_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus nlaedit_add_actionclip_exec(bContext *C, wmOperator *op)
 {
   Main *bmain = CTX_data_main(C);
   bAnimContext ac;
@@ -756,7 +760,7 @@ void NLA_OT_actionclip_add(wmOperatorType *ot)
   ot->description =
       "Add an Action-Clip strip (i.e. an NLA Strip referencing an Action) to the active track";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = nlaedit_add_actionclip_invoke;
   ot->exec = nlaedit_add_actionclip_exec;
   ot->poll = nlaop_poll_tweakmode_off;
@@ -780,7 +784,7 @@ void NLA_OT_actionclip_add(wmOperatorType *ot)
  * Add a new transition strip between selected strips.
  * \{ */
 
-static int nlaedit_add_transition_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus nlaedit_add_transition_exec(bContext *C, wmOperator *op)
 {
   bAnimContext ac;
 
@@ -836,7 +840,7 @@ static int nlaedit_add_transition_exec(bContext *C, wmOperator *op)
       }
 
       /* allocate new strip */
-      strip = MEM_cnew<NlaStrip>("NlaStrip");
+      strip = MEM_callocN<NlaStrip>("NlaStrip");
       BLI_insertlinkafter(&nlt->strips, s1, strip);
 
       /* set the type */
@@ -893,7 +897,7 @@ void NLA_OT_transition_add(wmOperatorType *ot)
   ot->idname = "NLA_OT_transition_add";
   ot->description = "Add a transition strip between two adjacent selected strips";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_add_transition_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -907,7 +911,7 @@ void NLA_OT_transition_add(wmOperatorType *ot)
 /** \name Add Sound Clip Operator
  * \{ */
 
-static int nlaedit_add_sound_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_add_sound_exec(bContext *C, wmOperator * /*op*/)
 {
   Main *bmain = CTX_data_main(C);
   bAnimContext ac;
@@ -983,7 +987,7 @@ void NLA_OT_soundclip_add(wmOperatorType *ot)
   ot->idname = "NLA_OT_soundclip_add";
   ot->description = "Add a strip for controlling when speaker plays its sound clip";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_add_sound_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -1000,7 +1004,7 @@ void NLA_OT_soundclip_add(wmOperatorType *ot)
  * \{ */
 
 /* add the specified action as new strip */
-static int nlaedit_add_meta_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_add_meta_exec(bContext *C, wmOperator * /*op*/)
 {
   bAnimContext ac;
 
@@ -1058,7 +1062,7 @@ void NLA_OT_meta_add(wmOperatorType *ot)
   ot->idname = "NLA_OT_meta_add";
   ot->description = "Add new meta-strips incorporating the selected strips";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_add_meta_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -1074,7 +1078,7 @@ void NLA_OT_meta_add(wmOperatorType *ot)
  * Separate out the strips held by the selected meta-strips.
  * \{ */
 
-static int nlaedit_remove_meta_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_remove_meta_exec(bContext *C, wmOperator * /*op*/)
 {
   bAnimContext ac;
 
@@ -1123,7 +1127,7 @@ void NLA_OT_meta_remove(wmOperatorType *ot)
   ot->idname = "NLA_OT_meta_remove";
   ot->description = "Separate out the strips held by the selected meta-strips";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_remove_meta_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -1140,7 +1144,7 @@ void NLA_OT_meta_remove(wmOperatorType *ot)
  * putting them on new tracks above the one the originals were housed in.
  * \{ */
 
-static int nlaedit_duplicate_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus nlaedit_duplicate_exec(bContext *C, wmOperator *op)
 {
   bAnimContext ac;
 
@@ -1221,7 +1225,9 @@ static int nlaedit_duplicate_exec(bContext *C, wmOperator *op)
   return OPERATOR_CANCELLED;
 }
 
-static int nlaedit_duplicate_invoke(bContext *C, wmOperator *op, const wmEvent * /*event*/)
+static wmOperatorStatus nlaedit_duplicate_invoke(bContext *C,
+                                                 wmOperator *op,
+                                                 const wmEvent * /*event*/)
 {
   nlaedit_duplicate_exec(C, op);
 
@@ -1235,7 +1241,7 @@ void NLA_OT_duplicate(wmOperatorType *ot)
   ot->idname = "NLA_OT_duplicate";
   ot->description = "Duplicate selected NLA-Strips, adding the new strips to new track(s)";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = nlaedit_duplicate_invoke;
   ot->exec = nlaedit_duplicate_exec;
   ot->poll = nlaop_poll_tweakmode_off;
@@ -1259,7 +1265,7 @@ void NLA_OT_duplicate(wmOperatorType *ot)
  * Deletes the selected NLA-Strips.
  * \{ */
 
-static int nlaedit_delete_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_delete_exec(bContext *C, wmOperator * /*op*/)
 {
   bAnimContext ac;
 
@@ -1333,7 +1339,7 @@ void NLA_OT_delete(wmOperatorType *ot)
   ot->idname = "NLA_OT_delete";
   ot->description = "Delete selected strips";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_delete_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -1427,7 +1433,7 @@ static void nlaedit_split_strip_meta(NlaTrack *nlt, NlaStrip *strip)
 
 /* ----- */
 
-static int nlaedit_split_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_split_exec(bContext *C, wmOperator * /*op*/)
 {
   bAnimContext ac;
 
@@ -1496,7 +1502,7 @@ void NLA_OT_split(wmOperatorType *ot)
   ot->idname = "NLA_OT_split";
   ot->description = "Split selected strips at their midpoints";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_split_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -1514,7 +1520,7 @@ void NLA_OT_split(wmOperatorType *ot)
  * Toggles whether strips are muted or not.
  * \{ */
 
-static int nlaedit_toggle_mute_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_toggle_mute_exec(bContext *C, wmOperator * /*op*/)
 {
   bAnimContext ac;
 
@@ -1565,7 +1571,7 @@ void NLA_OT_mute_toggle(wmOperatorType *ot)
   ot->idname = "NLA_OT_mute_toggle";
   ot->description = "Mute or un-mute selected strips";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_toggle_mute_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -1581,7 +1587,7 @@ void NLA_OT_mute_toggle(wmOperatorType *ot)
  * Tries to exchange strips within their owner tracks.
  * \{ */
 
-static int nlaedit_swap_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus nlaedit_swap_exec(bContext *C, wmOperator *op)
 {
   bAnimContext ac;
 
@@ -1754,7 +1760,7 @@ void NLA_OT_swap(wmOperatorType *ot)
   ot->idname = "NLA_OT_swap";
   ot->description = "Swap order of selected strips within tracks";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_swap_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -1770,7 +1776,7 @@ void NLA_OT_swap(wmOperatorType *ot)
  * Tries to move the selected strips into the track above if possible.
  * \{ */
 
-static int nlaedit_move_up_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_move_up_exec(bContext *C, wmOperator * /*op*/)
 {
   bAnimContext ac;
 
@@ -1845,7 +1851,7 @@ void NLA_OT_move_up(wmOperatorType *ot)
   ot->idname = "NLA_OT_move_up";
   ot->description = "Move selected strips up a track if there's room";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_move_up_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -1861,7 +1867,7 @@ void NLA_OT_move_up(wmOperatorType *ot)
  * Tries to move the selected strips into the track above if possible.
  * \{ */
 
-static int nlaedit_move_down_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_move_down_exec(bContext *C, wmOperator * /*op*/)
 {
   bAnimContext ac;
 
@@ -1936,7 +1942,7 @@ void NLA_OT_move_down(wmOperatorType *ot)
   ot->idname = "NLA_OT_move_down";
   ot->description = "Move selected strips down a track if there's room";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_move_down_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -1952,7 +1958,7 @@ void NLA_OT_move_down(wmOperatorType *ot)
  * Recalculate the extents of the action ranges used for the selected strips.
  * \{ */
 
-static int nlaedit_sync_actlen_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus nlaedit_sync_actlen_exec(bContext *C, wmOperator *op)
 {
   bAnimContext ac;
 
@@ -2021,7 +2027,7 @@ void NLA_OT_action_sync_length(wmOperatorType *ot)
   ot->description =
       "Synchronize the length of the referenced Action with the length used in the strip";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_sync_actlen_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -2044,7 +2050,7 @@ void NLA_OT_action_sync_length(wmOperatorType *ot)
  * Ensure that each strip has its own action.
  * \{ */
 
-static int nlaedit_make_single_user_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_make_single_user_exec(bContext *C, wmOperator * /*op*/)
 {
   Main *bmain = CTX_data_main(C);
   bAnimContext ac;
@@ -2109,7 +2115,9 @@ static int nlaedit_make_single_user_exec(bContext *C, wmOperator * /*op*/)
   return OPERATOR_FINISHED;
 }
 
-static int nlaedit_make_single_user_invoke(bContext *C, wmOperator *op, const wmEvent * /*event*/)
+static wmOperatorStatus nlaedit_make_single_user_invoke(bContext *C,
+                                                        wmOperator *op,
+                                                        const wmEvent * /*event*/)
 {
   if (RNA_boolean_get(op->ptr, "confirm")) {
     return WM_operator_confirm_ex(
@@ -2131,7 +2139,7 @@ void NLA_OT_make_single_user(wmOperatorType *ot)
   ot->idname = "NLA_OT_make_single_user";
   ot->description = "Make linked action local to each strip";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = nlaedit_make_single_user_invoke;
   ot->exec = nlaedit_make_single_user_exec;
   ot->poll = nlaop_poll_tweakmode_off;
@@ -2164,7 +2172,7 @@ static short bezt_apply_nlamapping(KeyframeEditData *ked, BezTriple *bezt)
   return 0;
 }
 
-static int nlaedit_apply_scale_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_apply_scale_exec(bContext *C, wmOperator * /*op*/)
 {
   Main *bmain = CTX_data_main(C);
   bAnimContext ac;
@@ -2267,7 +2275,7 @@ void NLA_OT_apply_scale(wmOperatorType *ot)
   ot->idname = "NLA_OT_apply_scale";
   ot->description = "Apply scaling of selected strips to their referenced Actions";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_apply_scale_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -2283,7 +2291,7 @@ void NLA_OT_apply_scale(wmOperatorType *ot)
  * Reset the scaling of the selected strips to 1.0f.
  * \{ */
 
-static int nlaedit_clear_scale_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus nlaedit_clear_scale_exec(bContext *C, wmOperator * /*op*/)
 {
   bAnimContext ac;
 
@@ -2333,7 +2341,7 @@ void NLA_OT_clear_scale(wmOperatorType *ot)
   ot->idname = "NLA_OT_clear_scale";
   ot->description = "Reset scaling of selected strips";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nlaedit_clear_scale_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -2360,7 +2368,7 @@ static const EnumPropertyItem prop_nlaedit_snap_types[] = {
     {0, nullptr, 0, nullptr, nullptr},
 };
 
-static int nlaedit_snap_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus nlaedit_snap_exec(bContext *C, wmOperator *op)
 {
   bAnimContext ac;
 
@@ -2496,7 +2504,7 @@ void NLA_OT_snap(wmOperatorType *ot)
   ot->idname = "NLA_OT_snap";
   ot->description = "Move start of strips to specified time";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = WM_menu_invoke;
   ot->exec = nlaedit_snap_exec;
   ot->poll = nlaop_poll_tweakmode_off;
@@ -2554,7 +2562,7 @@ static const EnumPropertyItem *nla_fmodifier_itemf(bContext *C,
   return item;
 }
 
-static int nla_fmodifier_add_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus nla_fmodifier_add_exec(bContext *C, wmOperator *op)
 {
   bAnimContext ac;
 
@@ -2640,7 +2648,7 @@ void NLA_OT_fmodifier_add(wmOperatorType *ot)
   ot->idname = "NLA_OT_fmodifier_add";
   ot->description = "Add F-Modifier to the active/selected NLA-Strips";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->invoke = WM_menu_invoke;
   ot->exec = nla_fmodifier_add_exec;
   ot->poll = nlaop_poll_tweakmode_off;
@@ -2667,7 +2675,7 @@ void NLA_OT_fmodifier_add(wmOperatorType *ot)
 /** \name Copy F-Modifiers Operator
  * \{ */
 
-static int nla_fmodifier_copy_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus nla_fmodifier_copy_exec(bContext *C, wmOperator *op)
 {
   bAnimContext ac;
   ListBase anim_data = {nullptr, nullptr};
@@ -2721,7 +2729,7 @@ void NLA_OT_fmodifier_copy(wmOperatorType *ot)
   ot->idname = "NLA_OT_fmodifier_copy";
   ot->description = "Copy the F-Modifier(s) of the active NLA-Strip";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nla_fmodifier_copy_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
@@ -2744,7 +2752,7 @@ void NLA_OT_fmodifier_copy(wmOperatorType *ot)
 /** \name Paste F-Modifiers Operator
  * \{ */
 
-static int nla_fmodifier_paste_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus nla_fmodifier_paste_exec(bContext *C, wmOperator *op)
 {
   bAnimContext ac;
   ListBase anim_data = {nullptr, nullptr};
@@ -2814,7 +2822,7 @@ void NLA_OT_fmodifier_paste(wmOperatorType *ot)
   ot->idname = "NLA_OT_fmodifier_paste";
   ot->description = "Add copied F-Modifiers to the selected NLA-Strips";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = nla_fmodifier_paste_exec;
   ot->poll = nlaop_poll_tweakmode_off;
 
