@@ -31,7 +31,7 @@
 #include "WM_api.hh"
 #include "WM_types.hh"
 
-#include "ANIM_bone_collections.hh"
+#include "ANIM_armature.hh"
 
 #include "tree/tree_element_seq.hh"
 
@@ -55,7 +55,7 @@ void ED_outliner_select_sync_from_pose_bone_tag(bContext *C)
   wm->outliner_sync_select_dirty |= WM_OUTLINER_SYNC_SELECT_FROM_POSE_BONE;
 }
 
-void ED_outliner_select_sync_from_sequence_tag(bContext *C)
+void ED_outliner_select_sync_from_sequence_tag(const bContext *C)
 {
   wmWindowManager *wm = CTX_wm_manager(C);
   wm->outliner_sync_select_dirty |= WM_OUTLINER_SYNC_SELECT_FROM_SEQUENCE;
@@ -254,7 +254,7 @@ static void outliner_select_sync_to_strip(Scene *scene, const TreeElement *te)
   Strip *strip = &te_strip->get_strip();
 
   if (tselem->flag & TSE_ACTIVE) {
-    SEQ_select_active_set(scene, strip);
+    seq::select_active_set(scene, strip);
   }
 
   if (tselem->flag & TSE_SELECTED) {
@@ -503,7 +503,7 @@ static void get_sync_select_active_data(const bContext *C, SyncSelectActiveData
   active_data->object = BKE_view_layer_active_object_get(view_layer);
   active_data->edit_bone = CTX_data_active_bone(C);
   active_data->pose_channel = CTX_data_active_pose_bone(C);
-  active_data->strip = SEQ_select_active_get(scene);
+  active_data->strip = seq::select_active_get(scene);
 }
 
 void outliner_sync_selection(const bContext *C,
