@@ -20,7 +20,7 @@ using namespace blender::gpu;
 namespace blender::gpu {
 
 /* Counter for active command buffers. */
-int MTLCommandBufferManager::num_active_cmd_bufs = 0;
+volatile std::atomic<int> MTLCommandBufferManager::num_active_cmd_bufs_in_system = 0;
 
 /* -------------------------------------------------------------------- */
 /** \name MTLCommandBuffer initialization and render coordination.
@@ -50,18 +50,16 @@ id<MTLCommandBuffer> MTLCommandBufferManager::ensure_begin()
      *
      * NOTE: We currently stall until completion of GPU work upon ::submit if we have reached the
      * in-flight command buffer limit. */
-    BLI_assert(MTLCommandBufferManager::num_active_cmd_bufs <
+    BLI_assert(MTLCommandBufferManager::num_active_cmd_bufs_in_system <
                GHOST_ContextCGL::max_command_buffer_count);
 
-    if (@available(macos 11.0, *)) {
-      if (G.debug & G_DEBUG_GPU) {
-        /* Debug: Enable Advanced Errors for GPU work execution. */
-        MTLCommandBufferDescriptor *desc = [[MTLCommandBufferDescriptor alloc] init];
-        desc.errorOptions = MTLCommandBufferErrorOptionEncoderExecutionStatus;
-        desc.retainedReferences = YES;
-        BLI_assert(context_.queue != nil);
-        active_command_buffer_ = [context_.queue commandBufferWithDescriptor:desc];
-      }
+    if (G.debug & G_DEBUG_GPU) {
+      /* Debug: Enable Advanced Errors for GPU work execution. */
+      MTLCommandBufferDescriptor *desc = [[MTLCommandBufferDescriptor alloc] init];
+      desc.errorOptions = MTLCommandBufferErrorOptionEncoderExecutionStatus;
+      desc.retainedReferences = YES;
+      BLI_assert(context_.queue != nil);
+      active_command_buffer_ = [context_.queue commandBufferWithDescriptor:desc];
     }
 
     /* Ensure command buffer is created if debug command buffer unavailable. */
@@ -70,7 +68,7 @@ id<MTLCommandBuffer> MTLCommandBufferManager::ensure_begin()
     }
 
     [active_command_buffer_ retain];
-    MTLCommandBufferManager::num_active_cmd_bufs++;
+    context_.main_command_buffer.inc_active_command_buffer_count();
 
     /* Ensure we begin new Scratch Buffer if we are on a new frame. */
     MTLScratchBufferManager &mem = context_.memory_manager;
@@ -92,6 +90,12 @@ bool MTLCommandBufferManager::submit(bool wait)
 {
   /* Skip submission if command buffer is empty. */
   if (empty_ || active_command_buffer_ == nil) {
+    if (wait) {
+      /* Wait for any previously submitted work on this context to complete.
+       * (The wait function will yield so may need reworking if this hits a
+       * performance critical path which is sensitive to CPU<->GPU latency) */
+      wait_until_active_command_buffers_complete();
+    }
     return false;
   }
 
@@ -125,7 +129,7 @@ bool MTLCommandBufferManager::submit(bool wait)
     [cmd_buffer_ref release];
 
     /* Decrement count. */
-    MTLCommandBufferManager::num_active_cmd_bufs--;
+    context_.main_command_buffer.dec_active_command_buffer_count();
   }];
 
   /* Submit command buffer to GPU. */
@@ -133,14 +137,14 @@ bool MTLCommandBufferManager::submit(bool wait)
 
   /* If we have too many active command buffers in flight, wait until completed to avoid running
    * out. We can increase */
-  if (MTLCommandBufferManager::num_active_cmd_bufs >=
+  if (MTLCommandBufferManager::num_active_cmd_bufs_in_system >=
       (GHOST_ContextCGL::max_command_buffer_count - 1))
   {
     wait = true;
     MTL_LOG_WARNING(
         "Maximum number of command buffers in flight. Host will wait until GPU work has "
         "completed. Consider increasing GHOST_ContextCGL::max_command_buffer_count or reducing "
-        "work fragmentation to better utilise system hardware. Command buffers are flushed upon "
+        "work fragmentation to better utilize system hardware. Command buffers are flushed upon "
         "GPUContext switches, this is the most common cause of excessive command buffer "
         "generation.");
   }
@@ -151,14 +155,12 @@ bool MTLCommandBufferManager::submit(bool wait)
 
     /* Command buffer execution debugging can return an error message if
      * execution has failed or encountered GPU-side errors. */
-    if (@available(macos 11.0, *)) {
-      if (G.debug & G_DEBUG_GPU) {
+    if (G.debug & G_DEBUG_GPU) {
 
-        NSError *error = [active_command_buffer_ error];
-        if (error != nil) {
-          NSLog(@"%@", error);
-          BLI_assert(false);
-        }
+      NSError *error = [active_command_buffer_ error];
+      if (error != nil) {
+        NSLog(@"%@", error);
+        BLI_assert(false);
       }
     }
   }
@@ -227,7 +229,7 @@ MTLFrameBuffer *MTLCommandBufferManager::get_active_framebuffer()
 
 /* Encoder and Pass management. */
 /* End currently active MTLCommandEncoder. */
-bool MTLCommandBufferManager::end_active_command_encoder()
+bool MTLCommandBufferManager::end_active_command_encoder(bool retain_framebuffers)
 {
 
   /* End active encoder if one is active. */
@@ -245,8 +247,10 @@ bool MTLCommandBufferManager::end_active_command_encoder()
         active_command_encoder_type_ = MTL_NO_COMMAND_ENCODER;
 
         /* Reset associated frame-buffer flag. */
-        active_frame_buffer_ = nullptr;
-        active_pass_descriptor_ = nullptr;
+        if (!retain_framebuffers) {
+          active_frame_buffer_ = nullptr;
+          active_pass_descriptor_ = nullptr;
+        }
         return true;
       }
 
@@ -286,7 +290,7 @@ bool MTLCommandBufferManager::end_active_command_encoder()
 }
 
 id<MTLRenderCommandEncoder> MTLCommandBufferManager::ensure_begin_render_command_encoder(
-    MTLFrameBuffer *ctx_framebuffer, bool force_begin, bool *new_pass)
+    MTLFrameBuffer *ctx_framebuffer, bool force_begin, bool *r_new_pass)
 {
   /* Ensure valid frame-buffer. */
   BLI_assert(ctx_framebuffer != nullptr);
@@ -356,11 +360,11 @@ id<MTLRenderCommandEncoder> MTLCommandBufferManager::ensure_begin_render_command
     render_pass_state_.reset_state();
 
     /* Return true as new pass started. */
-    *new_pass = true;
+    *r_new_pass = true;
   }
   else {
     /* No new pass. */
-    *new_pass = false;
+    *r_new_pass = false;
   }
 
   BLI_assert(active_render_command_encoder_ != nil);
@@ -487,10 +491,8 @@ bool MTLCommandBufferManager::do_break_submission()
     return ((current_draw_call_count_ > 30000) || (vertex_submitted_count_ > 100000000) ||
             (encoder_count_ > 25));
   }
-  else {
-    /* Apple Silicon is less efficient if splitting submissions. */
-    return false;
-  }
+  /* Apple Silicon is less efficient if splitting submissions. */
+  return false;
 }
 
 /** \} */
@@ -514,7 +516,7 @@ void MTLCommandBufferManager::push_debug_group(const char *name, int /*index*/)
       end_active_command_encoder();
     }
 
-    debug_group_stack.push_back(std::string(name));
+    debug_group_stack.emplace_back(name);
   }
 }
 
@@ -538,12 +540,12 @@ void MTLCommandBufferManager::pop_debug_group()
 #endif
 
     /* If we have pending debug groups, first pop the last pending one. */
-    if (debug_group_stack.size() > 0) {
+    if (!debug_group_stack.empty()) {
       debug_group_stack.pop_back();
     }
     else {
       /* Otherwise, close last active pushed group. */
-      if (debug_group_pushed_stack.size() > 0) {
+      if (!debug_group_pushed_stack.empty()) {
         debug_group_pushed_stack.pop_back();
 
         if (debug_group_pushed_stack.size() < uint(METAL_DEBUG_CAPTURE_MAX_NESTED_GROUPS)) {
@@ -578,78 +580,82 @@ bool MTLCommandBufferManager::insert_memory_barrier(eGPUBarrier barrier_bits,
                                                     eGPUStageBarrierBits before_stages,
                                                     eGPUStageBarrierBits after_stages)
 {
-  /* Only supporting Metal on 10.14 onward anyway - Check required for warnings. */
-  if (@available(macOS 10.14, *)) {
-
-    /* Apple Silicon does not support memory barriers for RenderCommandEncoder's.
-     * We do not currently need these due to implicit API guarantees.
-     * NOTE(Metal): MTLFence/MTLEvent may be required to synchronize work if
-     * untracked resources are ever used. */
-    if ([context_.device hasUnifiedMemory] &&
-        (active_command_encoder_type_ != MTL_COMPUTE_COMMAND_ENCODER))
-    {
-      return false;
+  /* Apple Silicon does not support memory barriers for RenderCommandEncoder's.
+   * We do not currently need these due to implicit API guarantees. However, render->render
+   * resource dependencies are only evaluated at RenderCommandEncoder boundaries due to work
+   * execution on TBDR architecture.
+   *
+   * NOTE: Render barriers are therefore inherently expensive. Where possible, opt for local
+   * synchronization using raster order groups, or, prefer compute to avoid subsequent passes
+   * re-loading pass attachments which are not needed. */
+  const bool is_tile_based_arch = (GPU_platform_architecture() == GPU_ARCHITECTURE_TBDR);
+  if (is_tile_based_arch) {
+    if (active_command_encoder_type_ == MTL_RENDER_COMMAND_ENCODER) {
+      /* Break render pass to ensure final pass results are visible to subsequent calls. */
+      end_active_command_encoder();
+      return true;
     }
+    /* Skip all barriers for compute and blit passes as Metal will resolve these dependencies. */
+    return false;
+  }
 
-    /* Resolve scope. */
-    MTLBarrierScope scope = 0;
-    if (barrier_bits & GPU_BARRIER_SHADER_IMAGE_ACCESS || barrier_bits & GPU_BARRIER_TEXTURE_FETCH)
-    {
-      bool is_compute = (active_command_encoder_type_ != MTL_RENDER_COMMAND_ENCODER);
-      scope |= (is_compute ? 0 : MTLBarrierScopeRenderTargets) | MTLBarrierScopeTextures;
-    }
-    if (barrier_bits & GPU_BARRIER_SHADER_STORAGE ||
-        barrier_bits & GPU_BARRIER_VERTEX_ATTRIB_ARRAY ||
-        barrier_bits & GPU_BARRIER_ELEMENT_ARRAY || barrier_bits & GPU_BARRIER_UNIFORM ||
-        barrier_bits & GPU_BARRIER_BUFFER_UPDATE)
-    {
-      scope = scope | MTLBarrierScopeBuffers;
-    }
+  /* Resolve scope. */
+  MTLBarrierScope scope = 0;
+  if (barrier_bits & GPU_BARRIER_SHADER_IMAGE_ACCESS || barrier_bits & GPU_BARRIER_TEXTURE_FETCH) {
+    bool is_compute = (active_command_encoder_type_ != MTL_RENDER_COMMAND_ENCODER);
+    scope |= (is_compute ? 0 : MTLBarrierScopeRenderTargets) | MTLBarrierScopeTextures;
+  }
+  if (barrier_bits & GPU_BARRIER_SHADER_STORAGE ||
+      barrier_bits & GPU_BARRIER_VERTEX_ATTRIB_ARRAY || barrier_bits & GPU_BARRIER_ELEMENT_ARRAY ||
+      barrier_bits & GPU_BARRIER_UNIFORM || barrier_bits & GPU_BARRIER_BUFFER_UPDATE)
+  {
+    scope = scope | MTLBarrierScopeBuffers;
+  }
 
-    if (scope != 0) {
-      /* Issue barrier based on encoder. */
-      switch (active_command_encoder_type_) {
-        case MTL_NO_COMMAND_ENCODER:
-        case MTL_BLIT_COMMAND_ENCODER: {
-          /* No barrier to be inserted. */
-          return false;
+  if (scope != 0) {
+    /* Issue barrier based on encoder. */
+    switch (active_command_encoder_type_) {
+      case MTL_NO_COMMAND_ENCODER:
+      case MTL_BLIT_COMMAND_ENCODER: {
+        /* No barrier to be inserted. */
+        return false;
+      }
+
+      /* Rendering. */
+      case MTL_RENDER_COMMAND_ENCODER: {
+        /* Currently flagging both stages -- can use bits above to filter on stage type --
+         * though full barrier is safe for now. */
+        MTLRenderStages before_stage_flags = 0;
+        MTLRenderStages after_stage_flags = 0;
+        if (before_stages & GPU_BARRIER_STAGE_VERTEX &&
+            !(before_stages & GPU_BARRIER_STAGE_FRAGMENT))
+        {
+          before_stage_flags = before_stage_flags | MTLRenderStageVertex;
+        }
+        if (before_stages & GPU_BARRIER_STAGE_FRAGMENT) {
+          before_stage_flags = before_stage_flags | MTLRenderStageFragment;
+        }
+        if (after_stages & GPU_BARRIER_STAGE_VERTEX) {
+          after_stage_flags = after_stage_flags | MTLRenderStageVertex;
+        }
+        if (after_stages & GPU_BARRIER_STAGE_FRAGMENT) {
+          after_stage_flags = MTLRenderStageFragment;
         }
 
-        /* Rendering. */
-        case MTL_RENDER_COMMAND_ENCODER: {
-          /* Currently flagging both stages -- can use bits above to filter on stage type --
-           * though full barrier is safe for now. */
-          MTLRenderStages before_stage_flags = 0;
-          MTLRenderStages after_stage_flags = 0;
-          if (before_stages & GPU_BARRIER_STAGE_VERTEX &&
-              !(before_stages & GPU_BARRIER_STAGE_FRAGMENT)) {
-            before_stage_flags = before_stage_flags | MTLRenderStageVertex;
-          }
-          if (before_stages & GPU_BARRIER_STAGE_FRAGMENT) {
-            before_stage_flags = before_stage_flags | MTLRenderStageFragment;
-          }
-          if (after_stages & GPU_BARRIER_STAGE_VERTEX) {
-            after_stage_flags = after_stage_flags | MTLRenderStageVertex;
-          }
-          if (after_stages & GPU_BARRIER_STAGE_FRAGMENT) {
-            after_stage_flags = MTLRenderStageFragment;
-          }
+        id<MTLRenderCommandEncoder> rec = this->get_active_render_command_encoder();
+        BLI_assert(rec != nil);
+        [rec memoryBarrierWithScope:scope
+                        afterStages:after_stage_flags
+                       beforeStages:before_stage_flags];
+        return true;
+      }
 
-          id<MTLRenderCommandEncoder> rec = this->get_active_render_command_encoder();
-          BLI_assert(rec != nil);
-          [rec memoryBarrierWithScope:scope
-                          afterStages:after_stage_flags
-                         beforeStages:before_stage_flags];
-          return true;
-        }
-
-        /* Compute. */
-        case MTL_COMPUTE_COMMAND_ENCODER: {
-          id<MTLComputeCommandEncoder> rec = this->get_active_compute_command_encoder();
-          BLI_assert(rec != nil);
-          [rec memoryBarrierWithScope:scope];
-          return true;
-        }
+      /* Compute. */
+      case MTL_COMPUTE_COMMAND_ENCODER: {
+        id<MTLComputeCommandEncoder> rec = this->get_active_compute_command_encoder();
+        BLI_assert(rec != nil);
+        [rec memoryBarrierWithScope:scope];
+        return true;
       }
     }
   }
@@ -664,6 +670,7 @@ void MTLCommandBufferManager::encode_signal_event(id<MTLEvent> event, uint64_t s
   BLI_assert(cmd_buf);
   this->end_active_command_encoder();
   [cmd_buf encodeSignalEvent:event value:signal_value];
+  register_encoder_counters();
 }
 
 void MTLCommandBufferManager::encode_wait_for_event(id<MTLEvent> event, uint64_t signal_value)
@@ -673,6 +680,7 @@ void MTLCommandBufferManager::encode_wait_for_event(id<MTLEvent> event, uint64_t
   BLI_assert(cmd_buf);
   this->end_active_command_encoder();
   [cmd_buf encodeWaitForEvent:event value:signal_value];
+  register_encoder_counters();
 }
 
 /** \} */
@@ -771,8 +779,6 @@ void MTLComputeState::bind_compute_texture(id<MTLTexture> tex, uint slot)
     id<MTLComputeCommandEncoder> rec = this->cmd.get_active_compute_command_encoder();
     BLI_assert(rec != nil);
     [rec setTexture:tex atIndex:slot];
-    [rec useResource:tex
-               usage:MTLResourceUsageRead | MTLResourceUsageWrite | MTLResourceUsageSample];
 
     this->cached_compute_texture_bindings[slot].metal_texture = tex;
   }
@@ -964,10 +970,7 @@ void MTLRenderPassState::bind_fragment_buffer(id<MTLBuffer> buffer,
   }
 }
 
-void MTLComputeState::bind_compute_buffer(id<MTLBuffer> buffer,
-                                          uint64_t buffer_offset,
-                                          uint index,
-                                          bool writeable)
+void MTLComputeState::bind_compute_buffer(id<MTLBuffer> buffer, uint64_t buffer_offset, uint index)
 {
   BLI_assert(index >= 0 && index < MTL_MAX_BUFFER_BINDINGS);
   BLI_assert(buffer_offset >= 0);
@@ -989,9 +992,6 @@ void MTLComputeState::bind_compute_buffer(id<MTLBuffer> buffer,
       /* Bind Compute Buffer */
       [rec setBuffer:buffer offset:buffer_offset atIndex:index];
     }
-    [rec useResource:buffer
-               usage:((writeable) ? (MTLResourceUsageRead | MTLResourceUsageWrite) :
-                                    MTLResourceUsageRead)];
 
     /* Update Bind-state cache */
     this->cached_compute_buffer_bindings[index].is_bytes = false;
@@ -1000,7 +1000,7 @@ void MTLComputeState::bind_compute_buffer(id<MTLBuffer> buffer,
   }
 }
 
-void MTLRenderPassState::bind_vertex_bytes(void *bytes, uint64_t length, uint index)
+void MTLRenderPassState::bind_vertex_bytes(const void *bytes, uint64_t length, uint index)
 {
   /* Bytes always updated as source data may have changed. */
   BLI_assert(index >= 0 && index < MTL_MAX_BUFFER_BINDINGS);
@@ -1025,7 +1025,7 @@ void MTLRenderPassState::bind_vertex_bytes(void *bytes, uint64_t length, uint in
   }
 }
 
-void MTLRenderPassState::bind_fragment_bytes(void *bytes, uint64_t length, uint index)
+void MTLRenderPassState::bind_fragment_bytes(const void *bytes, uint64_t length, uint index)
 {
   /* Bytes always updated as source data may have changed. */
   BLI_assert(index >= 0 && index < MTL_MAX_BUFFER_BINDINGS);
@@ -1050,7 +1050,7 @@ void MTLRenderPassState::bind_fragment_bytes(void *bytes, uint64_t length, uint
   }
 }
 
-void MTLComputeState::bind_compute_bytes(void *bytes, uint64_t length, uint index)
+void MTLComputeState::bind_compute_bytes(const void *bytes, uint64_t length, uint index)
 {
   /* Bytes always updated as source data may have changed. */
   BLI_assert(index >= 0 && index < MTL_MAX_BUFFER_BINDINGS);
@@ -1086,4 +1086,4 @@ void MTLComputeState::bind_pso(id<MTLComputePipelineState> pso)
 
 /** \} */
 
-}  // blender::gpu
+}  // namespace blender::gpu
