@@ -15,10 +15,6 @@
 
 #include "RNA_types.hh"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 struct bContext;
 
 /*
@@ -27,7 +23,3 @@ struct bContext;
 int collada_import(struct bContext *C, ImportSettings *import_settings);
 
 int collada_export(struct bContext *C, ExportSettings *export_settings);
-
-#ifdef __cplusplus
-}
-#endif
