@@ -12,6 +12,7 @@
 #include "MEM_guardedalloc.h"
 
 #include "BKE_context.hh"
+#include "BKE_library.hh"
 #include "BKE_modifier.hh"
 #include "BKE_screen.hh"
 
@@ -87,12 +88,12 @@ static void set_modifier_expand_flag(const bContext * /*C*/, Panel *panel, short
 /** \name Modifier Panel Layouts
  * \{ */
 
-void modifier_panel_end(uiLayout *layout, PointerRNA *ptr)
+void modifier_error_message_draw(uiLayout *layout, PointerRNA *ptr)
 {
   ModifierData *md = static_cast<ModifierData *>(ptr->data);
   if (md->error) {
-    uiLayout *row = uiLayoutRow(layout, false);
-    uiItemL(row, RPT_(md->error), ICON_ERROR);
+    uiLayout *row = &layout->row(false);
+    row->label(RPT_(md->error), ICON_ERROR);
   }
 }
 
@@ -130,13 +131,13 @@ void modifier_vgroup_ui(uiLayout *layout,
 {
   bool has_vertex_group = RNA_string_length(ptr, vgroup_prop.c_str()) != 0;
 
-  uiLayout *row = uiLayoutRow(layout, true);
+  uiLayout *row = &layout->row(true);
   uiItemPointerR(row, ptr, vgroup_prop, ob_ptr, "vertex_groups", text, ICON_GROUP_VERTEX);
   if (invert_vgroup_prop) {
-    uiLayout *sub = uiLayoutRow(row, true);
+    uiLayout *sub = &row->row(true);
     uiLayoutSetActive(sub, has_vertex_group);
     uiLayoutSetPropDecorate(sub, false);
-    uiItemR(sub, ptr, *invert_vgroup_prop, UI_ITEM_NONE, "", ICON_ARROW_LEFTRIGHT);
+    sub->prop(ptr, *invert_vgroup_prop, UI_ITEM_NONE, "", ICON_ARROW_LEFTRIGHT);
   }
 }
 
@@ -146,7 +147,7 @@ void modifier_grease_pencil_curve_header_draw(const bContext * /*C*/, Panel *pan
 
   PointerRNA *ptr = modifier_panel_get_property_pointers(panel, nullptr);
 
-  uiItemR(layout, ptr, "use_custom_curve", UI_ITEM_NONE, std::nullopt, ICON_NONE);
+  layout->prop(ptr, "use_custom_curve", UI_ITEM_NONE, std::nullopt, ICON_NONE);
 }
 
 void modifier_grease_pencil_curve_panel_draw(const bContext * /*C*/, Panel *panel)
@@ -221,44 +222,36 @@ static void modifier_ops_extra_draw(bContext *C, uiLayout *layout, void *md_v)
 
   /* Apply. */
   if (ob->type == OB_GREASE_PENCIL) {
-    uiItemO(layout,
-            CTX_IFACE_(BLT_I18NCONTEXT_OPERATOR_DEFAULT, "Apply (Active Keyframe)"),
-            ICON_CHECKMARK,
-            "OBJECT_OT_modifier_apply");
+    layout->op("OBJECT_OT_modifier_apply",
+               CTX_IFACE_(BLT_I18NCONTEXT_OPERATOR_DEFAULT, "Apply (Active Keyframe)"),
+               ICON_CHECKMARK);
 
-    uiItemFullO(layout,
-                "OBJECT_OT_modifier_apply",
-                IFACE_("Apply (All Keyframes)"),
-                ICON_KEYFRAME,
-                nullptr,
-                WM_OP_INVOKE_DEFAULT,
-                UI_ITEM_NONE,
-                &op_ptr);
+    op_ptr = layout->op("OBJECT_OT_modifier_apply",
+                        IFACE_("Apply (All Keyframes)"),
+                        ICON_KEYFRAME,
+                        WM_OP_INVOKE_DEFAULT,
+                        UI_ITEM_NONE);
     RNA_boolean_set(&op_ptr, "all_keyframes", true);
   }
   else {
-    uiItemO(layout,
-            CTX_IFACE_(BLT_I18NCONTEXT_OPERATOR_DEFAULT, "Apply"),
-            ICON_CHECKMARK,
-            "OBJECT_OT_modifier_apply");
+    layout->op("OBJECT_OT_modifier_apply",
+               CTX_IFACE_(BLT_I18NCONTEXT_OPERATOR_DEFAULT, "Apply"),
+               ICON_CHECKMARK);
   }
 
   /* Apply as shapekey. */
   if (BKE_modifier_is_same_topology(md) && !BKE_modifier_is_non_geometrical(md)) {
-    uiItemBooleanO(layout,
-                   CTX_IFACE_(BLT_I18NCONTEXT_OPERATOR_DEFAULT, "Apply as Shape Key"),
-                   ICON_SHAPEKEY_DATA,
-                   "OBJECT_OT_modifier_apply_as_shapekey",
-                   "keep_modifier",
-                   false);
+    PointerRNA op_ptr = layout->op(
+        "OBJECT_OT_modifier_apply_as_shapekey",
+        CTX_IFACE_(BLT_I18NCONTEXT_OPERATOR_DEFAULT, "Apply as Shape Key"),
+        ICON_SHAPEKEY_DATA);
+    RNA_boolean_set(&op_ptr, "keep_modifier", false);
 
-    uiItemBooleanO(layout,
-                   CTX_IFACE_(BLT_I18NCONTEXT_OPERATOR_DEFAULT, "Save as Shape Key"),
-                   ICON_NONE,
-                   "OBJECT_OT_modifier_apply_as_shapekey",
-                   "keep_modifier",
-                   true);
-    uiItemS(layout);
+    op_ptr = layout->op("OBJECT_OT_modifier_apply_as_shapekey",
+                        CTX_IFACE_(BLT_I18NCONTEXT_OPERATOR_DEFAULT, "Save as Shape Key"),
+                        ICON_NONE);
+    RNA_boolean_set(&op_ptr, "keep_modifier", true);
+    layout->separator();
   }
 
   /* Duplicate. */
@@ -269,56 +262,45 @@ static void modifier_ops_extra_draw(bContext *C, uiLayout *layout, void *md_v)
             eModifierType_Cloth,
             eModifierType_Fluid))
   {
-    uiItemO(layout,
-            CTX_IFACE_(BLT_I18NCONTEXT_OPERATOR_DEFAULT, "Duplicate"),
-            ICON_DUPLICATE,
-            "OBJECT_OT_modifier_copy");
+    layout->op("OBJECT_OT_modifier_copy",
+               CTX_IFACE_(BLT_I18NCONTEXT_OPERATOR_DEFAULT, "Duplicate"),
+               ICON_DUPLICATE);
   }
 
-  uiItemO(layout,
-          CTX_IFACE_(BLT_I18NCONTEXT_OPERATOR_DEFAULT, "Copy to Selected"),
-          0,
-          "OBJECT_OT_modifier_copy_to_selected");
+  layout->op("OBJECT_OT_modifier_copy_to_selected",
+             CTX_IFACE_(BLT_I18NCONTEXT_OPERATOR_DEFAULT, "Copy to Selected"),
+             0);
 
-  uiItemS(layout);
+  layout->separator();
 
   /* Move to first. */
-  uiItemFullO(layout,
-              "OBJECT_OT_modifier_move_to_index",
-              IFACE_("Move to First"),
-              ICON_TRIA_UP,
-              nullptr,
-              WM_OP_INVOKE_DEFAULT,
-              UI_ITEM_NONE,
-              &op_ptr);
+  op_ptr = layout->op("OBJECT_OT_modifier_move_to_index",
+                      IFACE_("Move to First"),
+                      ICON_TRIA_UP,
+                      WM_OP_INVOKE_DEFAULT,
+                      UI_ITEM_NONE);
   RNA_int_set(&op_ptr, "index", 0);
 
   /* Move to last. */
-  uiItemFullO(layout,
-              "OBJECT_OT_modifier_move_to_index",
-              IFACE_("Move to Last"),
-              ICON_TRIA_DOWN,
-              nullptr,
-              WM_OP_INVOKE_DEFAULT,
-              UI_ITEM_NONE,
-              &op_ptr);
+  op_ptr = layout->op("OBJECT_OT_modifier_move_to_index",
+                      IFACE_("Move to Last"),
+                      ICON_TRIA_DOWN,
+                      WM_OP_INVOKE_DEFAULT,
+                      UI_ITEM_NONE);
   RNA_int_set(&op_ptr, "index", BLI_listbase_count(&ob->modifiers) - 1);
 
-  uiItemS(layout);
+  layout->separator();
 
-  uiItemR(layout, &ptr, "use_pin_to_last", UI_ITEM_NONE, std::nullopt, ICON_NONE);
+  layout->prop(&ptr, "use_pin_to_last", UI_ITEM_NONE, std::nullopt, ICON_NONE);
 
   if (md->type == eModifierType_Nodes) {
-    uiItemS(layout);
-    uiItemFullO(layout,
-                "OBJECT_OT_geometry_nodes_move_to_nodes",
-                std::nullopt,
-                ICON_NONE,
-                nullptr,
-                WM_OP_INVOKE_DEFAULT,
-                UI_ITEM_NONE,
-                &op_ptr);
-    uiItemR(layout, &ptr, "show_group_selector", UI_ITEM_NONE, std::nullopt, ICON_NONE);
+    layout->separator();
+    op_ptr = layout->op("OBJECT_OT_geometry_nodes_move_to_nodes",
+                        std::nullopt,
+                        ICON_NONE,
+                        WM_OP_INVOKE_DEFAULT,
+                        UI_ITEM_NONE);
+    layout->prop(&ptr, "show_group_selector", UI_ITEM_NONE, std::nullopt, ICON_NONE);
   }
 }
 
@@ -339,35 +321,31 @@ static void modifier_panel_header(const bContext *C, Panel *panel)
   int index = BLI_findindex(&ob->modifiers, md);
 
   /* Modifier Icon. */
-  sub = uiLayoutRow(layout, true);
-  uiLayoutSetEmboss(sub, UI_EMBOSS_NONE);
+  sub = &layout->row(true);
+  uiLayoutSetEmboss(sub, blender::ui::EmbossType::None);
   if (mti->is_disabled && mti->is_disabled(scene, md, false)) {
     uiLayoutSetRedAlert(sub, true);
   }
-  uiItemStringO(sub,
-                "",
-                RNA_struct_ui_icon(ptr->type),
-                "OBJECT_OT_modifier_set_active",
-                "modifier",
-                md->name);
+  PointerRNA op_ptr = sub->op("OBJECT_OT_modifier_set_active", "", RNA_struct_ui_icon(ptr->type));
+  RNA_string_set(&op_ptr, "modifier", md->name);
 
-  row = uiLayoutRow(layout, true);
+  row = &layout->row(true);
 
   /* Modifier Name.
    * Count how many buttons are added to the header to check if there is enough space. */
   int buttons_number = 0;
-  name_row = uiLayoutRow(row, true);
+  name_row = &row->row(true);
 
   /* Display mode switching buttons. */
   if (ob->type == OB_MESH) {
     int last_cage_index;
     int cage_index = BKE_modifiers_get_cage_index(scene, ob, &last_cage_index, false);
     if (BKE_modifier_supports_cage(scene, md) && (index <= last_cage_index)) {
-      sub = uiLayoutRow(row, true);
+      sub = &row->row(true);
       if (index < cage_index || !BKE_modifier_couldbe_cage(scene, md)) {
         uiLayoutSetActive(sub, false);
       }
-      uiItemR(sub, ptr, "show_on_cage", UI_ITEM_NONE, "", ICON_NONE);
+      sub->prop(ptr, "show_on_cage", UI_ITEM_NONE, "", ICON_NONE);
       buttons_number++;
     }
   } /* Tessellation point for curve-typed objects. */
@@ -375,7 +353,7 @@ static void modifier_panel_header(const bContext *C, Panel *panel)
     /* Smooth modifier can work with tessellated curves only (works on mesh edges explicitly). */
     if (md->type == eModifierType_Smooth) {
       /* Add button (appearing to be OFF) and add tip why this can't be changed. */
-      sub = uiLayoutRow(row, true);
+      sub = &row->row(true);
       uiBlock *block = uiLayoutGetBlock(sub);
       static int apply_on_spline_always_off_hack = 0;
       uiBut *but = uiDefIconButBitI(block,
@@ -399,7 +377,7 @@ static void modifier_panel_header(const bContext *C, Panel *panel)
     else if (ELEM(md->type, eModifierType_Hook, eModifierType_Softbody, eModifierType_MeshDeform))
     {
       /* Add button (appearing to be ON) and add tip why this can't be changed. */
-      sub = uiLayoutRow(row, true);
+      sub = &row->row(true);
       uiBlock *block = uiLayoutGetBlock(sub);
       static int apply_on_spline_always_on_hack = eModifierMode_ApplyOnSpline;
       uiBut *but = uiDefIconButBitI(block,
@@ -421,56 +399,60 @@ static void modifier_panel_header(const bContext *C, Panel *panel)
     }
     else if (mti->type != ModifierTypeType::Constructive) {
       /* Constructive modifiers tessellates curve before applying. */
-      uiItemR(row, ptr, "use_apply_on_spline", UI_ITEM_NONE, "", ICON_NONE);
+      row->prop(ptr, "use_apply_on_spline", UI_ITEM_NONE, "", ICON_NONE);
       buttons_number++;
     }
   }
   /* Collision and Surface are always enabled, hide buttons. */
   if (!ELEM(md->type, eModifierType_Collision, eModifierType_Surface)) {
     if (mti->flags & eModifierTypeFlag_SupportsEditmode) {
-      sub = uiLayoutRow(row, true);
+      sub = &row->row(true);
       uiLayoutSetActive(sub, (md->mode & eModifierMode_Realtime));
-      uiItemR(sub, ptr, "show_in_editmode", UI_ITEM_NONE, "", ICON_NONE);
+      sub->prop(ptr, "show_in_editmode", UI_ITEM_NONE, "", ICON_NONE);
       buttons_number++;
     }
-    uiItemR(row, ptr, "show_viewport", UI_ITEM_NONE, "", ICON_NONE);
-    uiItemR(row, ptr, "show_render", UI_ITEM_NONE, "", ICON_NONE);
+    row->prop(ptr, "show_viewport", UI_ITEM_NONE, "", ICON_NONE);
+    row->prop(ptr, "show_render", UI_ITEM_NONE, "", ICON_NONE);
     buttons_number += 2;
   }
 
   /* Extra operators menu. */
-  uiItemMenuF(row, "", ICON_DOWNARROW_HLT, modifier_ops_extra_draw, md);
+  row->menu_fn("", ICON_DOWNARROW_HLT, modifier_ops_extra_draw, md);
 
   /* Delete button. */
   if (modifier_can_delete(md) && !modifier_is_simulation(md)) {
-    sub = uiLayoutRow(row, false);
-    uiLayoutSetEmboss(sub, UI_EMBOSS_NONE);
-    uiItemO(sub, "", ICON_X, "OBJECT_OT_modifier_remove");
+    sub = &row->row(false);
+    uiLayoutSetEmboss(sub, blender::ui::EmbossType::None);
+    sub->op("OBJECT_OT_modifier_remove", "", ICON_X);
     buttons_number++;
   }
 
   /* Switch context buttons. */
   if (modifier_is_simulation(md) == 1) {
-    uiItemStringO(
-        row, "", ICON_PROPERTIES, "WM_OT_properties_context_change", "context", "PHYSICS");
+    PointerRNA op_ptr = row->op("WM_OT_properties_context_change", "", ICON_PROPERTIES);
+    if (!RNA_pointer_is_null(&op_ptr)) {
+      RNA_string_set(&op_ptr, "context", "PHYSICS");
+    }
     buttons_number++;
   }
   else if (modifier_is_simulation(md) == 2) {
-    uiItemStringO(
-        row, "", ICON_PROPERTIES, "WM_OT_properties_context_change", "context", "PARTICLES");
+    PointerRNA op_ptr = row->op("WM_OT_properties_context_change", "", ICON_PROPERTIES);
+    if (!RNA_pointer_is_null(&op_ptr)) {
+      RNA_string_set(&op_ptr, "context", "PARTICLES");
+    }
     buttons_number++;
   }
 
   bool display_name = (panel->sizex / UI_UNIT_X - buttons_number > 5) || (panel->sizex == 0);
   if (display_name) {
-    uiItemR(name_row, ptr, "name", UI_ITEM_NONE, "", ICON_NONE);
+    name_row->prop(ptr, "name", UI_ITEM_NONE, "", ICON_NONE);
   }
   else {
     uiLayoutSetAlignment(row, UI_LAYOUT_ALIGN_RIGHT);
   }
 
   /* Extra padding for delete button. */
-  uiItemS(layout);
+  layout->separator();
 }
 
 /** \} */
@@ -481,7 +463,7 @@ static void modifier_panel_header(const bContext *C, Panel *panel)
 
 PanelType *modifier_panel_register(ARegionType *region_type, ModifierType type, PanelDrawFn draw)
 {
-  PanelType *panel_type = MEM_cnew<PanelType>(__func__);
+  PanelType *panel_type = MEM_callocN<PanelType>(__func__);
 
   BKE_modifier_type_panel_id(type, panel_type->idname);
   STRNCPY(panel_type->label, "");
@@ -513,7 +495,7 @@ PanelType *modifier_subpanel_register(ARegionType *region_type,
                                       PanelDrawFn draw,
                                       PanelType *parent)
 {
-  PanelType *panel_type = MEM_cnew<PanelType>(__func__);
+  PanelType *panel_type = MEM_callocN<PanelType>(__func__);
 
   BLI_assert(parent != nullptr);
   SNPRINTF(panel_type->idname, "%s_%s", parent->idname, name);
